import 'dart:convert';
import 'package:flutter/cupertino.dart';
import 'package:flutter_agnes/src/http_client/interfaces/i_model.dart';
import 'package:flutter_agnes/src/http_client/models/generic_model.dart';
import 'package:flutter_agnes/src/http_client/types_basics/types_basics.dart';
import 'package:http/http.dart' as http;

class RequestHelpers<T extends IModel> {
  final ObjectCreator<T> objectCreator;
  final String token;
  GenericModel<T> generic;

  RequestHelpers({@required this.objectCreator, this.token}) {
    generic = new GenericModel<T>(objectCreator);
  }

  T _getContent(http.Response response) {
    switch (response.statusCode) {
      case 200:
        var content = json.decode(response.body);
        return generic.getObject(content);
      case 401:
        throw json.decode(response.body);
      case 400:
        throw json.decode(response.body);
      default:
        throw json.decode(response.body);
    }
  }

  Future<T> getRequest(String url) async {
    try {
      print(url);
      Map<String, String> headers = {'Content-type': 'application/json; chrset=UTF-8'};
      if (token != null) headers.addAll({'Authorization': 'Bearer $token'});
      http.Response response = await http.get(Uri.parse(url), headers: headers);
      print('${response.statusCode} - ${response.body}');
      return _getContent(response);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  Future<T> postRequest(Map dataRow, String url) async {
    try {
      print(url);
      Map<String, String> headers = {'Content-type': 'application/json; chrset=UTF-8'};
      if (token != null) headers.addAll({'Authorization': 'Bearer $token'});
      http.Response response = await http.post(
        Uri.parse(url),
        body: json.encode(dataRow),
        headers: headers,
      );
      print('${response.statusCode} - ${response.body}');
      return _getContent(response);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  Future<T> putRequest(Map<String, dynamic> dataRow, String url) async {
    try {
      print(url);
      Map<String, String> headers = {'Content-type': 'application/json; chrset=UTF-8'};
      if (token != null) headers.addAll({'Authorization': 'Bearer $token'});
      http.Response response = await http.put(
        Uri.parse(url),
        body: json.encode(dataRow),
        headers: headers,
      );
      print('${response.statusCode} - ${response.body}');
      return _getContent(response);
    } catch (e) {
      print(e);
      throw e;
    }
  }
}
