import 'package:flutter/material.dart';

class DropdownMenuItemCustomWidget<T> extends StatelessWidget {

  final String title;
  final IconData prefixIcon;
  final IconData suffixIcon;
  final Color colorIcon;
  final Color colorText;
  final Color background;
  final T value;
  final ValueChanged<T> onChanged;
  final List<DropdownMenuItem<T>> items;

  const DropdownMenuItemCustomWidget({
    this.prefixIcon = Icons.list,
    this.suffixIcon = Icons.arrow_drop_down,
    this.colorIcon = Colors.grey,
    this.colorText = Colors.black,
    this.background = const Color(0xfff3f3f4),
    this.items,
    this.title,
    this.onChanged,
    this.value,
    Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        Container(
          padding: EdgeInsets.only(left: 0),
          alignment: Alignment.topLeft,
          child: Text(
            title,
            style: TextStyle(color: colorText, fontSize: 14),
          ),
        ),
        Container(
            color: background,
            child: Row(
              children: [
                Container(
                  padding: EdgeInsets.only(left: 10, right: 15),
                  child: Icon(prefixIcon, color: colorIcon),
                ),
                Expanded(
                    child: DropdownButton<T>(
                      elevation: 16,
                      iconSize: 24,
                      isExpanded: true,
                      value: value,
                      items: items,
                      underline: Container(
                        height: 0,
                        color: background,
                      ),
                      icon: Icon(
                        suffixIcon,
                        color: colorIcon,
                      ),
                      onChanged: onChanged,
                    ))
              ],
            )),
      ],
    );
  }
}